<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/03/19
 * Time: 14.49
 */

namespace FiloBlu\RuleDayFromBirthday\Model\Rule\Condition\Customer;


use Magento\Customer\Api\CustomerRepositoryInterface;

class Birthday extends \Magento\Rule\Model\Condition\AbstractCondition
{

    /**
     * @var CustomerRepositoryInterface
     */
    protected $customerRepository;

    /**
     * Birthday constructor.
     * @param \Magento\Rule\Model\Condition\Context $context
     * @param CustomerRepositoryInterface $customerRepository
     * @param array $data
     */
    public function __construct(
        \Magento\Rule\Model\Condition\Context $context,
        CustomerRepositoryInterface $customerRepository,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->customerRepository = $customerRepository;
    }
    /**
     * Load attribute options
     *
     * @return $this
     */
    public function loadAttributeOptions()
    {
        $attributes = [
            'customer_birthday' => __("Days from customer's birthday")
        ];
        $this->setAttributeOption($attributes);
        return $this;
    }
    /**
     * Get input type
     * Possible values are: string, numeric, date, select, multiselect, grid, boolean
     *
     * @return string
     */
    public function getInputType()
    {
        return 'string';
    }
    /**
     * Get value element type
     *
     * We have the value element types as select, text
     * @return string
     */
    public function getValueElementType()
    {
        return 'text';
    }


    /**
     * @param int $customerId
     * @return string|null
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getCustomerDobById(int $customerId)
    {
        return $this->customerRepository->getById($customerId)->getDob();
    }


    /**
     * @param $dateString
     * @return \DateTime
     * @throws \Exception
     */
    protected function updateDate($dateString)
    {
        $suppliedDate = new \DateTime($dateString);
        $currentYear = (int)(new \DateTime())->format('Y');
        return (new \DateTime())->setDate($currentYear, (int)$suppliedDate->format('m'), (int)$suppliedDate->format('d'));
    }


    /**
     * Validate Customer Rule Condition
     *
     * @param \Magento\Framework\Model\AbstractModel $model
     * @return bool
     */
    public function validate(\Magento\Framework\Model\AbstractModel $model)
    {

        // Get the customer id of the current customer
        $customerId = (int)$model->getCustomerId();
        $dob = $this->getCustomerDobById($customerId);
        if(!is_null($dob))
        {
            $dob_now = $this->updateDate($dob);
            $today = new \DateTime();
            $val = $today->diff($dob_now)->format('%a');
            $model->setData('customer_birthday', $val);
            return parent::validate($model);
        }
        else
        {
            $dob_now = null;
            return false;
        }



    }
}